@@ -1,32 +1,91 @@
 import logging
 import os
+import time
+from mock import patch
+import pytest
+import kafka.codec
 
+import pytest
 from six.moves import xrange
+import six
 
+from . import unittest
 from kafka import (
-    KafkaConsumer, MultiProcessConsumer, SimpleConsumer, create_message
-)
-from kafka.common import (
-    ProduceRequest, ConsumerFetchSizeTooSmall, ConsumerTimeout,
-    OffsetOutOfRangeError
+    KafkaConsumer, MultiProcessConsumer, SimpleConsumer, create_message,
+    create_gzip_message, KafkaProducer
 )
 from kafka.consumer.base import MAX_FETCH_BUFFER_SIZE_BYTES
+from kafka.errors import (
+    ConsumerFetchSizeTooSmall, OffsetOutOfRangeError, UnsupportedVersionError,
+    KafkaTimeoutError, UnsupportedCodecError
+)
+from kafka.structs import (
+    ProduceRequestPayload, TopicPartition, OffsetAndTimestamp
+)
 
+from test.conftest import version
 from test.fixtures import ZookeeperFixture, KafkaFixture
 from test.testutil import (
-    KafkaIntegrationTestCase, kafka_versions, random_string, Timer
+    KafkaIntegrationTestCase, kafka_versions, random_string, Timer,
+    send_messages
 )
 
 
+@pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
+def test_kafka_consumer(simple_client, topic, kafka_consumer_factory):
+    """Test KafkaConsumer
+    """
+    kafka_consumer = kafka_consumer_factory(auto_offset_reset='earliest')
+
+    send_messages(simple_client, topic, 0, range(0, 100))
+    send_messages(simple_client, topic, 1, range(100, 200))
+
+    cnt = 0
+    messages = {0: set(), 1: set()}
+    for message in kafka_consumer:
+        logging.debug("Consumed message %s", repr(message))
+        cnt += 1
+        messages[message.partition].add(message.offset)
+        if cnt >= 200:
+            break
+
+    assert len(messages[0]) == 100
+    assert len(messages[1]) == 100
+    kafka_consumer.close()
+
+
+@pytest.mark.skipif(not version(), reason="No KAFKA_VERSION set")
+def test_kafka_consumer_unsupported_encoding(
+        topic, kafka_producer_factory, kafka_consumer_factory):
+    # Send a compressed message
+    producer = kafka_producer_factory(compression_type="gzip")
+    fut = producer.send(topic, b"simple message" * 200)
+    fut.get(timeout=5)
+    producer.close()
+
+    # Consume, but with the related compression codec not available
+    with patch.object(kafka.codec, "has_gzip") as mocked:
+        mocked.return_value = False
+        consumer = kafka_consumer_factory(auto_offset_reset='earliest')
+        error_msg = "Libraries for gzip compression codec not found"
+        with pytest.raises(UnsupportedCodecError, match=error_msg):
+            consumer.poll(timeout_ms=2000)
+
+
 class TestConsumerIntegration(KafkaIntegrationTestCase):
+    maxDiff = None
+
     @classmethod
     def setUpClass(cls):
         if not os.environ.get('KAFKA_VERSION'):
             return
 
         cls.zk = ZookeeperFixture.instance()
-        cls.server1 = KafkaFixture.instance(0, cls.zk.host, cls.zk.port)
-        cls.server2 = KafkaFixture.instance(1, cls.zk.host, cls.zk.port)
+        chroot = random_string(10)
+        cls.server1 = KafkaFixture.instance(0, cls.zk,
+                                            zk_chroot=chroot)
+        cls.server2 = KafkaFixture.instance(1, cls.zk,
+                                            zk_chroot=chroot)
 
         cls.server = cls.server1 # Bootstrapping server
 
@@ -41,12 +100,18 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
     def send_messages(self, partition, messages):
         messages = [ create_message(self.msg(str(msg))) for msg in messages ]
-        produce = ProduceRequest(self.bytes_topic, partition, messages = messages)
+        produce = ProduceRequestPayload(self.topic, partition, messages = messages)
         resp, = self.client.send_produce_request([produce])
         self.assertEqual(resp.error, 0)
 
         return [ x.value for x in messages ]
 
+    def send_gzip_message(self, partition, messages):
+        message = create_gzip_message([(self.msg(str(msg)), None) for msg in messages])
+        produce = ProduceRequestPayload(self.topic, partition, messages = [message])
+        resp, = self.client.send_produce_request([produce])
+        self.assertEqual(resp.error, 0)
+
     def assert_message_count(self, messages, num_messages):
         # Make sure we got them all
         self.assertEqual(len(messages), num_messages)
@@ -60,10 +125,11 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
             kwargs['group'] = None
             kwargs['auto_commit'] = False
         else:
-            kwargs.setdefault('auto_commit', True)
+            kwargs.setdefault('group', None)
+            kwargs.setdefault('auto_commit', False)
 
         consumer_class = kwargs.pop('consumer', SimpleConsumer)
-        group = kwargs.pop('group', self.id().encode('utf-8'))
+        group = kwargs.pop('group', None)
         topic = kwargs.pop('topic', self.topic)
 
         if consumer_class in [SimpleConsumer, MultiProcessConsumer]:
@@ -78,7 +144,12 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
                                  **configs)
         return consumer
 
-    @kafka_versions("all")
+    def kafka_producer(self, **configs):
+        brokers = '%s:%d' % (self.server.host, self.server.port)
+        producer = KafkaProducer(
+            bootstrap_servers=brokers, **configs)
+        return producer
+
     def test_simple_consumer(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
@@ -90,7 +161,17 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions('all')
+    def test_simple_consumer_gzip(self):
+        self.send_gzip_message(0, range(0, 100))
+        self.send_gzip_message(1, range(100, 200))
+
+        # Start a consumer
+        consumer = self.consumer()
+
+        self.assert_message_count([ message for message in consumer ], 200)
+
+        consumer.stop()
+
     def test_simple_consumer_smallest_offset_reset(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
@@ -102,7 +183,6 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # messages from beginning.
         self.assert_message_count([message for message in consumer], 200)
 
-    @kafka_versions('all')
     def test_simple_consumer_largest_offset_reset(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
@@ -120,7 +200,6 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # Since the offset is set to largest we should read all the new messages.
         self.assert_message_count([message for message in consumer], 200)
 
-    @kafka_versions('all')
     def test_simple_consumer_no_reset(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
@@ -132,13 +211,13 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         with self.assertRaises(OffsetOutOfRangeError):
             consumer.get_message()
 
-    @kafka_versions("0.8.1", "0.8.1.1", "0.8.2.1")
+    @kafka_versions('>=0.8.1')
     def test_simple_consumer_load_initial_offsets(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
 
         # Create 1st consumer and change offsets
-        consumer = self.consumer()
+        consumer = self.consumer(group='test_simple_consumer_load_initial_offsets')
         self.assertEqual(consumer.offsets, {0: 0, 1: 0})
         consumer.offsets.update({0:51, 1:101})
         # Update counter after manual offsets update
@@ -146,10 +225,10 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         consumer.commit()
 
         # Create 2nd consumer and check initial offsets
-        consumer = self.consumer(auto_commit=False)
+        consumer = self.consumer(group='test_simple_consumer_load_initial_offsets',
+                                 auto_commit=False)
         self.assertEqual(consumer.offsets, {0: 51, 1: 101})
 
-    @kafka_versions("all")
     def test_simple_consumer__seek(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
@@ -180,7 +259,6 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions("all")
     def test_simple_consumer_blocking(self):
         consumer = self.consumer()
 
@@ -190,13 +268,14 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
             self.assert_message_count(messages, 0)
         self.assertGreaterEqual(t.interval, 1)
 
-        self.send_messages(0, range(0, 10))
+        self.send_messages(0, range(0, 5))
+        self.send_messages(1, range(5, 10))
 
         # Ask for 5 messages, 10 in queue. Get 5 back, no blocking
         with Timer() as t:
-            messages = consumer.get_messages(count=5, block=True, timeout=5)
+            messages = consumer.get_messages(count=5, block=True, timeout=3)
             self.assert_message_count(messages, 5)
-        self.assertLessEqual(t.interval, 1)
+        self.assertLess(t.interval, 3)
 
         # Ask for 10 messages, get 5 back, block 1 second
         with Timer() as t:
@@ -206,7 +285,8 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         # Ask for 10 messages, 5 in queue, ask to block for 1 message or 1
         # second, get 5 back, no blocking
-        self.send_messages(0, range(0, 5))
+        self.send_messages(0, range(0, 3))
+        self.send_messages(1, range(3, 5))
         with Timer() as t:
             messages = consumer.get_messages(count=10, block=1, timeout=1)
             self.assert_message_count(messages, 5)
@@ -214,7 +294,6 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions("all")
     def test_simple_consumer_pending(self):
         # make sure that we start with no pending messages
         consumer = self.consumer()
@@ -242,7 +321,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         self.assertEquals(set([0, 1]), set([pending_part1, pending_part2]))
         consumer.stop()
 
-    @kafka_versions("all")
+    @unittest.skip('MultiProcessConsumer deprecated and these tests are flaky')
     def test_multi_process_consumer(self):
         # Produce 100 messages to partitions 0 and 1
         self.send_messages(0, range(0, 100))
@@ -254,7 +333,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions("all")
+    @unittest.skip('MultiProcessConsumer deprecated and these tests are flaky')
     def test_multi_process_consumer_blocking(self):
         consumer = self.consumer(consumer = MultiProcessConsumer)
 
@@ -292,7 +371,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions("all")
+    @unittest.skip('MultiProcessConsumer deprecated and these tests are flaky')
     def test_multi_proc_pending(self):
         self.send_messages(0, range(0, 10))
         self.send_messages(1, range(10, 20))
@@ -308,13 +387,14 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions("0.8.1", "0.8.1.1", "0.8.2.1")
+    @unittest.skip('MultiProcessConsumer deprecated and these tests are flaky')
+    @kafka_versions('>=0.8.1')
     def test_multi_process_consumer_load_initial_offsets(self):
         self.send_messages(0, range(0, 10))
         self.send_messages(1, range(10, 20))
 
         # Create 1st consumer and change offsets
-        consumer = self.consumer()
+        consumer = self.consumer(group='test_multi_process_consumer_load_initial_offsets')
         self.assertEqual(consumer.offsets, {0: 0, 1: 0})
         consumer.offsets.update({0:5, 1:15})
         # Update counter after manual offsets update
@@ -323,10 +403,10 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         # Create 2nd consumer and check initial offsets
         consumer = self.consumer(consumer = MultiProcessConsumer,
+                                 group='test_multi_process_consumer_load_initial_offsets',
                                  auto_commit=False)
         self.assertEqual(consumer.offsets, {0: 5, 1: 15})
 
-    @kafka_versions("all")
     def test_large_messages(self):
         # Produce 10 "normal" size messages
         small_messages = self.send_messages(0, [ str(x) for x in range(10) ])
@@ -334,8 +414,14 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # Produce 10 messages that are large (bigger than default fetch size)
         large_messages = self.send_messages(0, [ random_string(5000) for x in range(10) ])
 
-        # Consumer should still get all of them
-        consumer = self.consumer()
+        # Brokers prior to 0.11 will return the next message
+        # if it is smaller than max_bytes (called buffer_size in SimpleConsumer)
+        # Brokers 0.11 and later that store messages in v2 format
+        # internally will return the next message only if the
+        # full MessageSet is smaller than max_bytes.
+        # For that reason, we set the max buffer size to a little more
+        # than the size of all large messages combined
+        consumer = self.consumer(max_buffer_size=60000)
 
         expected_messages = set(small_messages + large_messages)
         actual_messages = set([ x.message.value for x in consumer ])
@@ -343,7 +429,6 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         consumer.stop()
 
-    @kafka_versions("all")
     def test_huge_messages(self):
         huge_message, = self.send_messages(0, [
             create_message(random_string(MAX_FETCH_BUFFER_SIZE_BYTES + 10)),
@@ -352,7 +437,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # Create a consumer with the default buffer size
         consumer = self.consumer()
 
-        # This consumer failes to get the message
+        # This consumer fails to get the message
         with self.assertRaises(ConsumerFetchSizeTooSmall):
             consumer.get_message(False, 0.1)
 
@@ -374,13 +459,15 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         big_consumer.stop()
 
-    @kafka_versions("0.8.1", "0.8.1.1", "0.8.2.1")
+    @kafka_versions('>=0.8.1')
     def test_offset_behavior__resuming_behavior(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
 
         # Start a consumer
         consumer1 = self.consumer(
+            group='test_offset_behavior__resuming_behavior',
+            auto_commit=True,
             auto_commit_every_t = None,
             auto_commit_every_n = 20,
         )
@@ -391,6 +478,8 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
 
         # The total offset across both partitions should be at 180
         consumer2 = self.consumer(
+            group='test_offset_behavior__resuming_behavior',
+            auto_commit=True,
             auto_commit_every_t = None,
             auto_commit_every_n = 20,
         )
@@ -401,7 +490,8 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         consumer1.stop()
         consumer2.stop()
 
-    @kafka_versions("0.8.1", "0.8.1.1", "0.8.2.1")
+    @unittest.skip('MultiProcessConsumer deprecated and these tests are flaky')
+    @kafka_versions('>=0.8.1')
     def test_multi_process_offset_behavior__resuming_behavior(self):
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
@@ -409,6 +499,8 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # Start a consumer
         consumer1 = self.consumer(
             consumer=MultiProcessConsumer,
+            group='test_multi_process_offset_behavior__resuming_behavior',
+            auto_commit=True,
             auto_commit_every_t = None,
             auto_commit_every_n = 20,
             )
@@ -426,6 +518,8 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # The total offset across both partitions should be at 180
         consumer2 = self.consumer(
             consumer=MultiProcessConsumer,
+            group='test_multi_process_offset_behavior__resuming_behavior',
+            auto_commit=True,
             auto_commit_every_t = None,
             auto_commit_every_n = 20,
             )
@@ -437,7 +531,6 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         consumer2.stop()
 
     # TODO: Make this a unit test -- should not require integration
-    @kafka_versions("all")
     def test_fetch_buffer_size(self):
 
         # Test parameters (see issue 135 / PR 136)
@@ -455,37 +548,20 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         messages = [ message for message in consumer ]
         self.assertEqual(len(messages), 2)
 
-    @kafka_versions("all")
-    def test_kafka_consumer(self):
-        self.send_messages(0, range(0, 100))
-        self.send_messages(1, range(100, 200))
-
-        # Start a consumer
-        consumer = self.kafka_consumer(auto_offset_reset='smallest',
-                                       consumer_timeout_ms=5000)
-        n = 0
-        messages = {0: set(), 1: set()}
-        logging.debug("kafka consumer offsets: %s" % consumer.offsets())
-        for m in consumer:
-            logging.debug("Consumed message %s" % repr(m))
-            n += 1
-            messages[m.partition].add(m.offset)
-            if n >= 200:
-                break
-
-        self.assertEqual(len(messages[0]), 100)
-        self.assertEqual(len(messages[1]), 100)
-
-    @kafka_versions("all")
     def test_kafka_consumer__blocking(self):
         TIMEOUT_MS = 500
-        consumer = self.kafka_consumer(auto_offset_reset='smallest',
+        consumer = self.kafka_consumer(auto_offset_reset='earliest',
+                                       enable_auto_commit=False,
                                        consumer_timeout_ms=TIMEOUT_MS)
 
+        # Manual assignment avoids overhead of consumer group mgmt
+        consumer.unsubscribe()
+        consumer.assign([TopicPartition(self.topic, 0)])
+
         # Ask for 5 messages, nothing in queue, block 500ms
         with Timer() as t:
-            with self.assertRaises(ConsumerTimeout):
-                msg = consumer.next()
+            with self.assertRaises(StopIteration):
+                msg = next(consumer)
         self.assertGreaterEqual(t.interval, TIMEOUT_MS / 1000.0 )
 
         self.send_messages(0, range(0, 10))
@@ -494,7 +570,7 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         messages = set()
         with Timer() as t:
             for i in range(5):
-                msg = consumer.next()
+                msg = next(consumer)
                 messages.add((msg.partition, msg.offset))
         self.assertEqual(len(messages), 5)
         self.assertLess(t.interval, TIMEOUT_MS / 1000.0 )
@@ -502,52 +578,211 @@ class TestConsumerIntegration(KafkaIntegrationTestCase):
         # Ask for 10 messages, get 5 back, block 500ms
         messages = set()
         with Timer() as t:
-            with self.assertRaises(ConsumerTimeout):
+            with self.assertRaises(StopIteration):
                 for i in range(10):
-                    msg = consumer.next()
+                    msg = next(consumer)
                     messages.add((msg.partition, msg.offset))
         self.assertEqual(len(messages), 5)
         self.assertGreaterEqual(t.interval, TIMEOUT_MS / 1000.0 )
+        consumer.close()
 
-    @kafka_versions("0.8.1", "0.8.1.1", "0.8.2.1")
+    @kafka_versions('>=0.8.1')
     def test_kafka_consumer__offset_commit_resume(self):
-        GROUP_ID = random_string(10).encode('utf-8')
+        GROUP_ID = random_string(10)
 
         self.send_messages(0, range(0, 100))
         self.send_messages(1, range(100, 200))
 
         # Start a consumer
         consumer1 = self.kafka_consumer(
-            group_id = GROUP_ID,
-            auto_commit_enable = True,
-            auto_commit_interval_ms = None,
-            auto_commit_interval_messages = 20,
-            auto_offset_reset='smallest',
+            group_id=GROUP_ID,
+            enable_auto_commit=True,
+            auto_commit_interval_ms=100,
+            auto_offset_reset='earliest',
         )
 
-        # Grab the first 195 messages
+        # Grab the first 180 messages
         output_msgs1 = []
-        for _ in xrange(195):
-            m = consumer1.next()
+        for _ in xrange(180):
+            m = next(consumer1)
             output_msgs1.append(m)
-            consumer1.task_done(m)
-        self.assert_message_count(output_msgs1, 195)
+        self.assert_message_count(output_msgs1, 180)
+        consumer1.close()
 
         # The total offset across both partitions should be at 180
         consumer2 = self.kafka_consumer(
-            group_id = GROUP_ID,
-            auto_commit_enable = True,
-            auto_commit_interval_ms = None,
-            auto_commit_interval_messages = 20,
-            consumer_timeout_ms = 100,
-            auto_offset_reset='smallest',
+            group_id=GROUP_ID,
+            enable_auto_commit=True,
+            auto_commit_interval_ms=100,
+            auto_offset_reset='earliest',
         )
 
         # 181-200
         output_msgs2 = []
-        with self.assertRaises(ConsumerTimeout):
-            while True:
-                m = consumer2.next()
-                output_msgs2.append(m)
+        for _ in xrange(20):
+            m = next(consumer2)
+            output_msgs2.append(m)
         self.assert_message_count(output_msgs2, 20)
-        self.assertEqual(len(set(output_msgs1) & set(output_msgs2)), 15)
+        self.assertEqual(len(set(output_msgs1) | set(output_msgs2)), 200)
+        consumer2.close()
+
+    @kafka_versions('>=0.10.1')
+    def test_kafka_consumer_max_bytes_simple(self):
+        self.send_messages(0, range(100, 200))
+        self.send_messages(1, range(200, 300))
+
+        # Start a consumer
+        consumer = self.kafka_consumer(
+            auto_offset_reset='earliest', fetch_max_bytes=300)
+        seen_partitions = set([])
+        for i in range(10):
+            poll_res = consumer.poll(timeout_ms=100)
+            for partition, msgs in six.iteritems(poll_res):
+                for msg in msgs:
+                    seen_partitions.add(partition)
+
+        # Check that we fetched at least 1 message from both partitions
+        self.assertEqual(
+            seen_partitions, set([
+                TopicPartition(self.topic, 0), TopicPartition(self.topic, 1)]))
+        consumer.close()
+
+    @kafka_versions('>=0.10.1')
+    def test_kafka_consumer_max_bytes_one_msg(self):
+        # We send to only 1 partition so we don't have parallel requests to 2
+        # nodes for data.
+        self.send_messages(0, range(100, 200))
+
+        # Start a consumer. FetchResponse_v3 should always include at least 1
+        # full msg, so by setting fetch_max_bytes=1 we should get 1 msg at a time
+        # But 0.11.0.0 returns 1 MessageSet at a time when the messages are
+        # stored in the new v2 format by the broker.
+        #
+        # DP Note: This is a strange test. The consumer shouldn't care
+        # how many messages are included in a FetchResponse, as long as it is
+        # non-zero. I would not mind if we deleted this test. It caused
+        # a minor headache when testing 0.11.0.0.
+        group = 'test-kafka-consumer-max-bytes-one-msg-' + random_string(5)
+        consumer = self.kafka_consumer(
+            group_id=group,
+            auto_offset_reset='earliest',
+            consumer_timeout_ms=5000,
+            fetch_max_bytes=1)
+
+        fetched_msgs = [next(consumer) for i in range(10)]
+        self.assertEqual(len(fetched_msgs), 10)
+        consumer.close()
+
+    @kafka_versions('>=0.10.1')
+    def test_kafka_consumer_offsets_for_time(self):
+        late_time = int(time.time()) * 1000
+        middle_time = late_time - 1000
+        early_time = late_time - 2000
+        tp = TopicPartition(self.topic, 0)
+
+        timeout = 10
+        kafka_producer = self.kafka_producer()
+        early_msg = kafka_producer.send(
+            self.topic, partition=0, value=b"first",
+            timestamp_ms=early_time).get(timeout)
+        late_msg = kafka_producer.send(
+            self.topic, partition=0, value=b"last",
+            timestamp_ms=late_time).get(timeout)
+
+        consumer = self.kafka_consumer()
+        offsets = consumer.offsets_for_times({tp: early_time})
+        self.assertEqual(len(offsets), 1)
+        self.assertEqual(offsets[tp].offset, early_msg.offset)
+        self.assertEqual(offsets[tp].timestamp, early_time)
+
+        offsets = consumer.offsets_for_times({tp: middle_time})
+        self.assertEqual(offsets[tp].offset, late_msg.offset)
+        self.assertEqual(offsets[tp].timestamp, late_time)
+
+        offsets = consumer.offsets_for_times({tp: late_time})
+        self.assertEqual(offsets[tp].offset, late_msg.offset)
+        self.assertEqual(offsets[tp].timestamp, late_time)
+
+        offsets = consumer.offsets_for_times({})
+        self.assertEqual(offsets, {})
+
+        # Out of bound timestamps check
+
+        offsets = consumer.offsets_for_times({tp: 0})
+        self.assertEqual(offsets[tp].offset, early_msg.offset)
+        self.assertEqual(offsets[tp].timestamp, early_time)
+
+        offsets = consumer.offsets_for_times({tp: 9999999999999})
+        self.assertEqual(offsets[tp], None)
+
+        # Beginning/End offsets
+
+        offsets = consumer.beginning_offsets([tp])
+        self.assertEqual(offsets, {
+            tp: early_msg.offset,
+        })
+        offsets = consumer.end_offsets([tp])
+        self.assertEqual(offsets, {
+            tp: late_msg.offset + 1
+        })
+        consumer.close()
+
+    @kafka_versions('>=0.10.1')
+    def test_kafka_consumer_offsets_search_many_partitions(self):
+        tp0 = TopicPartition(self.topic, 0)
+        tp1 = TopicPartition(self.topic, 1)
+
+        kafka_producer = self.kafka_producer()
+        send_time = int(time.time() * 1000)
+        timeout = 10
+        p0msg = kafka_producer.send(
+            self.topic, partition=0, value=b"XXX",
+            timestamp_ms=send_time).get(timeout)
+        p1msg = kafka_producer.send(
+            self.topic, partition=1, value=b"XXX",
+            timestamp_ms=send_time).get(timeout)
+
+        consumer = self.kafka_consumer()
+        offsets = consumer.offsets_for_times({
+            tp0: send_time,
+            tp1: send_time
+        })
+
+        self.assertEqual(offsets, {
+            tp0: OffsetAndTimestamp(p0msg.offset, send_time),
+            tp1: OffsetAndTimestamp(p1msg.offset, send_time)
+        })
+
+        offsets = consumer.beginning_offsets([tp0, tp1])
+        self.assertEqual(offsets, {
+            tp0: p0msg.offset,
+            tp1: p1msg.offset
+        })
+
+        offsets = consumer.end_offsets([tp0, tp1])
+        self.assertEqual(offsets, {
+            tp0: p0msg.offset + 1,
+            tp1: p1msg.offset + 1
+        })
+        consumer.close()
+
+    @kafka_versions('<0.10.1')
+    def test_kafka_consumer_offsets_for_time_old(self):
+        consumer = self.kafka_consumer()
+        tp = TopicPartition(self.topic, 0)
+
+        with self.assertRaises(UnsupportedVersionError):
+            consumer.offsets_for_times({tp: int(time.time())})
+
+    @kafka_versions('>=0.10.1')
+    def test_kafka_consumer_offsets_for_times_errors(self):
+        consumer = self.kafka_consumer(fetch_max_wait_ms=200,
+                                       request_timeout_ms=500)
+        tp = TopicPartition(self.topic, 0)
+        bad_tp = TopicPartition(self.topic, 100)
+
+        with self.assertRaises(ValueError):
+            consumer.offsets_for_times({tp: -1})
+
+        with self.assertRaises(KafkaTimeoutError):
+            consumer.offsets_for_times({bad_tp: 0})
